// Board specifc definitions for
// JUMA-PA100 RF amplifier
// Juha Niinikoski OH2NLT 22.11.2008

// Processor header
#include <p30f6014A.h>

// UART setup according clock config
#define BAUD				9600
#define SERIAL_MODE			0			// JUMA-PA100 / JUMA-TRX2 protocol

#define POLL_TIMER			8000		// TRX2 protocol query time (ms)
//#define MAX_POLL_TIMER	60000		// MAX = 1 min, 0 = Off
#define MAX_POLL_TIMER		30000		// MAX = 30 sceonds, 0 = Off

// Clock constants

//#define FCY				7500000UL	// 30MHz ext Osc / 4  = 7,5MHz
#define FCY					7372800UL	// 29,4912MHz ext Osc / 4  = 7,3728MHz

#define DLYCONST	((FCY/1000000UL) + 1)	// Constant for timer delay routines

// PA100 band select limits(kHz)

#define LM1					1500		// Invalid when under this limit
#define LM2					2650		// 1.8 MHz filter band when under this limit
#define LM4					5250		// 3.5 MHz filter band when under this limit
#define LM7					8500		// 7 MHz filter band when under this limit
#define LM10				12000		// 10 MHz filter band when under this limit
#define LM14				16000		// 14 MHz filter band when under this limit
#define LM18				19500		// 18 MHz filter band when under this limit
#define LM21				22500		// 21 MHz filter band when under this limit
#define LM24				26000		// 21 MHz filter band when under this limit
#define LM28				30000		// 28 MHz filter band when under this limit

// Select algorithm parameters
#define F_SAMPLES			20			// Number of samples, 1ms/sample
#define	V_SAMPLES			5			// Required number of non-zero samples for decision

// Band selector
#define MIN_BAND			1			// Band selector limits
#define MAX_BAND			9
#define NOT_KNOWN			10			// When Auto / Frequency unknown
#define OUT_OF_BAND			0			// Out-Of-Band indicator

// LCD parameters & factory defaults
#define DEFAULT_CONTRAST	2000
#define MAX_CONTRAST 		3500
#define DEFAULT_BL			300
#define MAX_BL				1100

// Panel buttons
//#define BUTTON_DEBOUNCE 	200			// Debounce delay(ms) "original"
#define BUTTON_DEBOUNCE		100			// Debounce delay(ms) for fast fingers(oh7sv)
#define LONG_PUSH			1000		// Long button push
#define VERY_LONG_PUSH		3000		// Very long push
#define POWER_OFF 			1200		// Power-off push
#define DEFAULT_BEEP_LEN	50			// Button beep length 50ms, long = * 10
#define BLINK_RATE 			500			// Alarm text blink rate (ms)
#define ALARM_BEEP_LEN 		30			// Alarm beep length

// Encoder simulator with UP and DOWN buttons
#define REP 				80			// UP / DOWN buttons repeat interval
#define REP_DLY 			500			// Delay before repeat starts

// Alarm flags
// Alarm bits
#define SWR_AL				(1<<1)
#define CURR_AL				(1<<2)
#define TEMP_AL				(1<<3)

// Alarm trip limits
#define TEMP_LIMITC			70			// Degrees C / Display units
#define TEMP_LIMITF			160			// Degrees F / Display units
#define MIN_TEMPC			50			// Adjustment limits, C
#define MIN_TEMPF			120			// Adjustment limits, F
#define MAX_TEMPC			100
#define MAX_TEMPF			212

#define SWR_LIMIT			300			// Default value 0,01 units, 500 = 5.0
#define MAX_SWR				1000		// Max setup value - Changed to 10:1

// Power meter
// Fast attack, slow release parameters
#define PEAK_SHOW_TIME		350			// Freeze after new peak value(ms)
#define DECAY_TIME			50			// Power meter decay tiimer(ms)
#define DECAY_RATE			1			// Decay rate 

// ADC values
// Channel definitions
#define ID_CUR				9			// PA Drain current channel
#define BATT_CH				10			// Battery channel
#define B_TYPE				11			// Board type
#define Y_817				11			// Yaesu 817 band data
#define REV_PWR				12			// Reverse power channel
#define FWD_PWR				13			// Forward power
#define TEMP				14			// PA Temperature

// Board type limits
#define B1					819			// 1V
#define B2					1638		// 2V, F-sense board type = 1...2V
#define B3					2457		// 3V
#define B4					3276		// 4V
#define B5					4095		// 5V	

// Tone generator constants, FCYMHz / tone(Hz) * 2

// Some harmony sounds, FCY = 7,5MHz
/*
#define HZ392_01			9566		// G, off
#define HZ466_85			8044		// B-flat, push button standard tone
#define HZ587_31			6385		// D, fast tune tone
#define HZ698_45			5369		// F, CW side tone
*/

// Some harmony sounds, FCY = 7,3728MHz
/*
#define HZ392_01			9404		// G, off
#define HZ466_85			7896		// B-flat, push button standard tone
#define HZ587_31			6277		// D, fast tune tone
#define HZ698_45			5278		// F, CW side tone
*/

// Some harmony sounds * 2, FCY = 7,3728MHz
#define HZ392_01			4702		// G, off
#define HZ466_85			3948		// B-flat, push button standard tone
#define HZ587_31			6138		// D, fast tune tone
#define HZ698_45			2639		// F, CW side tone

// Some harmony sounds * 4, FCY = 7,3728MHz
/*
#define HZ392_01			2351		// G, off
#define HZ466_85			1974		// B-flat, push button standard tone
#define HZ587_31			1569		// D, fast tune tone
#define HZ698_45			1319		// F, Alarm tone
*/

// Meter scaling factors, factory defaults
#define AD_SCALE_FACTOR		1221UL		// Based on 5V regulator voltage (5000 / 4096)
#define BATT_MULT 			5250UL		// Battery voltage = ADC * BATT_MULT / 1000000

//#define ID_MULT 			6613UL		// Default value for Drain current scaling, ID * 100 = ADC * ID_MULT / 65536
#define ID_MULT 			1260UL		// Default value for Drain current scaling, ID = ADC * ID_MULT / 100000
 
#define	S_MTR_MULT 			1920UL		// Default value for S-meter scaling, S-meter = ADC * S_MTR_MULT / 65536, 48 = max meter = 2.0V FS

//#define REV_PWR_MULT 		23UL		// Default value for Reverse Power scaling
//#define FWD_PWR_MULT 		23UL		// Default value for Forward Power scaling, P * 100 =  ADC^2 * FWD_PWR_MULT / 65536, 10W = 1000

#define	REV_PWR_MULT 		1200UL		// Default value for Reverse Power scaling
#define	FWD_PWR_MULT 		1200UL		// Default value for Forward Power scaling, P = ((ADC^2) / 10) * FWD_PWR_MULT / 10000000 

#define PWR_GRAPH_MULT 		18UL		// Power bar graph, 48 = max = 10W, ADC^2 * PWR_GRAPH_MULT / 1048576
#define F_VOLT_GRAPH_MULT 	1874UL	 	// Power bar graph, 48 = max = 10W, ADC * F_VOLT_GRAPH_MULT / 65536

#define T_MULTC				4000UL		// Degrees C temperature scaling. T degC = ADC * T_MULTC / 65536
#define T_MULTF				7200UL		// Degrees F temperature scaling. T degF = ADC * T_MULTF / 65536

// Fan speed table, deg C
#define FAN_STARTC			40			// Fan start temp, low, C
#define FAN_STARTF			104			// Fan start temp, low, F
#define F_speed_2F			10			// Fan Speed 2, +10F
#define F_speed_2C			5			// Mid, low + 5 = 45
#define F_speed_3F			20			// Fan Speed 3, raw ADC value, +10C
#define F_speed_3C			10			// High, low + 10 = 55
#define MAX_FAN_STARTF		212			// Adjustment limit, F
#define MAX_FAN_STARTC		100			// Adjustment limit, C

// Power-on & calibration value addresses in the EEPROM
#define EEPAGE 				0x7F		// EEPROM address high part
#define EEDEF 				0xF000		// EEPROM default values storage area
#define EECAL 				0xF040		// EEPROM calibration values storage area
#define EE_FD_LOC 			0xF0F0		// Counter address in EEPROM

// PA100 board I/O definitions

// Port A Switches

#define INIT_TRISA 			0xFF3F		// PortA switch inputs
#define INIT_PORTA 			0x0000

#define AUTO				_RA12		// SW2, AUTO
#define DISP				_RA13		// SW1, FUNC / DISP
#define DN					_RA14		// SW8, DOWN button
#define UP 					_RA15		// SW5, UP button

// Port B ADC inputs

//#define INIT_TRISB 		0x7E0B		// PortB ADC and IDC pins = inputs
#define INIT_TRISB 			0x7E08
#define INIT_PORTB 			0x0000

#define IRQ_TEST			_LATB0		// Timing test, J19-4
#define MAIN_TEST			_LATB1		// Timing test, J19-5

#define TX_ON				_LATB4		// 1 = PA active, RF power on
#define KEY					_RB3		// 1 = TX Request

#define OC_CLR				_LATB5		// Over-current clear, 0 = clear

#define DB_4				_LATB6		// RF attenuators
#define DB_2				_LATB7

#define PTT_IN_IO			_RB3		// PTT input / TX on, 0 = active, Test

// Port C Switches & power control

#define INIT_TRISC			0x401E
#define INIT_PORTC			0x0000

#define OPER				_RC4		// SW3, OPER
#define BAND_DN				_RC3		// SW4, Band- button
#define BAND_UP				_RC1		// SW6, Band+ button

#define F_SENSE				_RC14		// F-Sense counter input
#define PWR_ON				_RC13		// Power-on

// Port D LCD & PWM out

#define INIT_TRISD			0x0001		// LCD & LED I/O, all outputs
#define INIT_PORTD			0x0000

// Power switch
#define PWR_SW				_RD0		// Power switch input, 1 = switch pushed

// Tone output
#define TONE_OUT			_LATD1		// Tone output
#define TONE_TRIS			_TRISD1		// Tone output tri-state control

// LCD signals
#define LCD_E				_LATD5		// LCD E signal

#define LCD_RW				_LATD6		// LCD R/W signal
#define LCD_RS				_LATD7		// LCD RS signal

#define LCD_DATA			LATD		// LCD 8-bit data bus out register
#define LCD_BUS				PORTD		// LCD 8-bit data bus state
#define LCD_TRIS			TRISD		// LCD data direction 

#define LCD_SHIFT			8			// LCD bits shifted from D0
#define LCD_MASK	(0x00FF<<LCD_SHIFT)	// LCD data lines at port

// PORT F RS-232

#define INIT_TRISF			0xFC7C
#define INIT_PORTF			0x0000

// PORT G FAN control

#define INIT_TRISG 			0x8000		// RG15 input, others outputs
#define INIT_PORTG 			0x0000

#define OC					_RG15		// Over-Current trip indicator from PA board

#define FAN1				_LATG2		// FAN High speed
#define FAN2				_LATG3		// FAN Low speed

#define TUNER				_LATG1		// Tuner relay

// Filter select I/O
#define M1_8				_LATG13		// 1.8 MHz
#define M3_5				_LATG12		// 3.5 MHz
#define M7					_LATG14		// 7 MHz
#define M10					_LATA7		// 10 MHz
#define M14_18				_LATA6		// 18-18 MHz
#define M21_28				_LATG0		// 21-28 MHz filter select

